#include <sta/hal/spi.hpp>

#ifdef STA_HAL_SPI_ENABLE

#include <sta/assert.hpp>
#include <sta/lang.hpp>


namespace sta
{
	static SpiSettings getHalSpiSettings(SPI_HandleTypeDef * handle)
	{
		SpiSettings settings;

		settings.mode        = getSpiMode(
			(handle->Init.CLKPolarity == SPI_POLARITY_LOW) ? SpiClkPolarity::LOW : SpiClkPolarity::HIGH,
			(handle->Init.CLKPhase == SPI_PHASE_1EDGE) ? SpiClkPhase::EDGE_1 : SpiClkPhase::EDGE_2
		);
		settings.dataSize    = (handle->Init.DataSize == SPI_DATASIZE_8BIT) ? SpiDataSize::BIT_8 : SpiDataSize::BIT_16;
		settings.bitOrder    = (handle->Init.FirstBit == SPI_FIRSTBIT_MSB) ? SpiBitOrder::MSB : SpiBitOrder::LSB;

		uint32_t prescaler = 1;
		switch (handle->Init.BaudRatePrescaler)
		{
		case SPI_BAUDRATEPRESCALER_2:
			settings.baudRatePrescaler = SpiBaudRatePrescaler::BRP_2;
			prescaler = 2;
			break;
		case SPI_BAUDRATEPRESCALER_4:
			settings.baudRatePrescaler = SpiBaudRatePrescaler::BRP_4;
			prescaler = 2;
			break;
		case SPI_BAUDRATEPRESCALER_8:
			settings.baudRatePrescaler = SpiBaudRatePrescaler::BRP_8;
			prescaler = 2;
			break;
		case SPI_BAUDRATEPRESCALER_16:
			settings.baudRatePrescaler = SpiBaudRatePrescaler::BRP_16;
			prescaler = 2;
			break;
		case SPI_BAUDRATEPRESCALER_32:
			settings.baudRatePrescaler = SpiBaudRatePrescaler::BRP_32;
			prescaler = 2;
			break;
		case SPI_BAUDRATEPRESCALER_64:
			settings.baudRatePrescaler = SpiBaudRatePrescaler::BRP_64;
			prescaler = 2;
			break;
		case SPI_BAUDRATEPRESCALER_128:
			settings.baudRatePrescaler = SpiBaudRatePrescaler::BRP_128;
			prescaler = 2;
			break;
		case SPI_BAUDRATEPRESCALER_256:
			settings.baudRatePrescaler = SpiBaudRatePrescaler::BRP_256;
			prescaler = 2;
			break;
		default:
			// Unreachable case
			STA_ASSERT_MSG(false, "Case for SPI_BAUDRATEPRESCALER not handled");
			STA_UNREACHABLE();
		}

		settings.clkSpeed = HAL_RCC_GetSysClockFreq() / prescaler;

		return settings;
	}


	HalSpiInterface::HalSpiInterface(SPI_HandleTypeDef * handle, Mutex * mutex /* = nullptr */)
		: SpiInterface(getHalSpiSettings(handle), mutex), handle_{handle}
	{
		STA_ASSERT(handle != nullptr);
	}


	void HalSpiInterface::transfer(uint8_t data)
	{
		HAL_SPI_Transmit(handle_, &data, 1, HAL_MAX_DELAY);
	}

	void HalSpiInterface::transfer(const uint8_t * buffer, size_t size)
	{
		STA_ASSERT(buffer != nullptr);

		HAL_SPI_Transmit(handle_, const_cast<uint8_t *>(buffer), size, HAL_MAX_DELAY);
	}

	void HalSpiInterface::transfer(const uint8_t * txBuffer, uint8_t * rxBuffer, size_t size)
	{
		STA_ASSERT(txBuffer != nullptr);
		STA_ASSERT(rxBuffer != nullptr);
		STA_ASSERT(size != 0);

		HAL_SPI_TransmitReceive(handle_, const_cast<uint8_t *>(txBuffer), rxBuffer, size, HAL_MAX_DELAY);
	}


	void HalSpiInterface::transfer16(uint16_t value)
	{
		static_assert(sizeof(uint16_t) == 2, "Unexpected uint16_t size");

		HAL_SPI_Transmit(handle_, reinterpret_cast<uint8_t *>(&value), 2, HAL_MAX_DELAY);
	}


	void HalSpiInterface::fill(uint8_t value, size_t count)
	{
		STA_ASSERT(count != 0);

		for (size_t i = 0; i < count; ++i)
		{
			transfer(value);
		}
	}

	void HalSpiInterface::fill32(uint32_t value, size_t count)
	{
		static_assert(sizeof(uint32_t) == 4, "Unexpected uint32_t size");
		STA_ASSERT(count != 0);

		for (size_t i = 0; i < count; ++i)
		{
			HAL_SPI_Transmit(handle_, reinterpret_cast<uint8_t *>(&value), 4, HAL_MAX_DELAY);
		}
	}


	void HalSpiInterface::receive(uint8_t * buffer, size_t size)
	{
		STA_ASSERT(buffer != nullptr);

		HAL_SPI_Receive(handle_, buffer, size, HAL_MAX_DELAY);
	}



	HalSpiDevice::HalSpiDevice(SpiInterface * intf, HalGpioPin csPin)
		: SpiDevice(intf), csPin_{csPin}
	{}

	void HalSpiDevice::select()
	{
		csPin_.setState(GpioPinState::LOW);
	}

	void HalSpiDevice::deselect()
	{
		csPin_.setState(GpioPinState::HIGH);
	}
} // namespace sta


#endif // STA_HAL_SPI_ENABLE
